# mkincl.tcl
# generate POV-Ray 'colours_x11.inc' from X Windows 'rgb.txt'.

# standard X colour database.
set infile /usr/share/X11/rgb.txt

proc i2f {v} {
  if { 255 == $v } {
    set res 1.0
  } elseif { !$v } {
    set res 0.0
  } else {
    set res [expr {(1.0 / 256) * $v}]
  }
  return $res
}

set ident colours_x11

# bail if no db or run already.
set outfile ${ident}.inc

if { ![file exists $infile] } {
  puts [format {oops, cannot find colour db '%s'.} $infile]
  exit 1
} elseif { [file exists $outfile] } {
  exit 1
}

# get colours.
set colours [list]
set re {^([[:digit:]]{1,3})\s+([[:digit:]]{1,3})\s+([[:digit:]]{1,3})\s+([[:alpha:]][[:alnum:]_ ]*)$}

set fp [open $infile r]
foreach line [split [read $fp] \n] {
  unset -nocomplain r g b name
  set line [string trim $line]
  if { ![regexp $re $line -> r g b name] } {
    continue
  }
  # adapt names, spaces to underscores, capitalise others.
  if { [regexp \\s $name] } {
    set name [regsub -all \\s+ $name _]
  } elseif { [string is lower [string index $name 0]] } {
    set name [string totitle $name]
  }
  lappend colours [list $name $r $g $b]
}
close $fp

# output colours.
set datim [clock format [clock seconds] -format {%d.%m.%Y %H%Mh}]
set guard ${ident}_include_temp
set n [llength $colours]

set fp [open $outfile w]
puts $fp [format {// POV-Ray 3.x  X Window System colours
// generated from '%s'
// on %s.

#ifndef(%s)

#declare %s = version;
#version 3.5;

#ifdef(View_POV_Include_Stack)
#debug "including %s\n"
#end 
} $infile $datim $guard $guard $outfile]
foreach c $colours {
  puts $fp [format {#declare %s = color red %.6f green %.6f blue %.6f;} \
          [lindex $c 0] [i2f [lindex $c 1]] [i2f [lindex $c 2]] [i2f [lindex $c 3]]]
}
puts $fp [format {
#version %s;

#end
// %d colour names.} $guard $n]
close $fp

# feedback.
puts [format {wrote '%s' with %d colour declarations.} $outfile $n]

exit 0

